@@ -7,6 +7,7 @@ import functools
 import sys
 import os
 import types
+import time
 
 from sphinx.directives import TocTree
 
@@ -192,16 +193,18 @@ project = 'Salt'
 copyright = '2016 SaltStack, Inc.'
 
 version = salt.version.__version__
-latest_release = '2015.8.8'  # latest release
-previous_release = '2015.5.10'  # latest release from previous branch
-previous_release_dir = '2015.5'  # path on web server for previous branch
-build_type = 'latest'  # latest, previous, develop, inactive
+latest_release = '2016.3.1'  # latest release
+previous_release = '2015.8.10'  # latest release from previous branch
+previous_release_dir = '2015.8'  # path on web server for previous branch
+next_release = ''  # next release
+next_release_dir = ''  # path on web server for next release branch
 
-# set release to 'version' for develop so sha is used
-# - otherwise -
-# set release to 'latest_release' or 'previous_release'
+today = time.strftime("%B %d, %Y") + " at " + time.strftime("%X %Z")
 
-release = latest_release  # version, latest_release, previous_release
+# < --- START do not merge these settings to other branches START ---> #
+build_type = 'previous'  # latest, previous, develop, next
+release = previous_release  # version, latest_release, previous_release
+# < --- END do not merge these settings to other branches END ---> #
 
 # Set google custom search engine
 
@@ -211,6 +214,8 @@ elif release.startswith('2014.7'):
     search_cx = '004624818632696854117:thhslradbru' # 2014.7
 elif release.startswith('2015.5'):
     search_cx = '004624818632696854117:ovogwef29do' # 2015.5
+elif release.startswith('2015.8'):
+    search_cx = '004624818632696854117:aw_tegffouy' # 2015.8
 else:
     search_cx = '004624818632696854117:haj7bjntf4s'  # develop
 
@@ -271,6 +276,7 @@ extlinks = {
     'blob': ('https://github.com/saltstack/salt/blob/%s/%%s' % 'develop', None),
     'download': ('https://cloud.github.com/downloads/saltstack/salt/%s', None),
     'issue': ('https://github.com/saltstack/salt/issues/%s', 'issue '),
+    'pull': ('https://github.com/saltstack/salt/pull/%s', 'PR '),
     'formula_url': ('https://github.com/saltstack-formulas/%s', ''),
 }
 
@@ -338,6 +344,7 @@ html_context = {
     'previous_release_dir': previous_release_dir,
     'search_cx': search_cx,
     'build_type': build_type,
+    'today': today,
 }
 
 html_use_index = True
