@@ -272,7 +272,7 @@ def is_file_ignored(opts, fname):
     return False
 
 
-def clear_lock(clear_func, lock_type, remote=None):
+def clear_lock(clear_func, role, remote=None, lock_type='update'):
     '''
     Function to allow non-fileserver functions to clear update locks
 
@@ -282,7 +282,7 @@ def clear_lock(clear_func, lock_type, remote=None):
         lists, one containing messages describing successfully cleared locks,
         and one containing messages describing errors encountered.
 
-    lock_type
+    role
         What type of lock is being cleared (gitfs, git_pillar, etc.). Used
         solely for logging purposes.
 
@@ -290,14 +290,16 @@ def clear_lock(clear_func, lock_type, remote=None):
         Optional string which should be used in ``func`` to pattern match so
         that a subset of remotes can be targeted.
 
+    lock_type : update
+        Which type of lock to clear
 
     Returns the return data from ``clear_func``.
     '''
-    msg = 'Clearing update lock for {0} remotes'.format(lock_type)
+    msg = 'Clearing {0} lock for {1} remotes'.format(lock_type, role)
     if remote:
         msg += ' matching {0}'.format(remote)
     log.debug(msg)
-    return clear_func(remote=remote)
+    return clear_func(remote=remote, lock_type=lock_type)
 
 
 class Fileserver(object):
@@ -314,25 +316,38 @@ class Fileserver(object):
         '''
         Return the backend list
         '''
-        ret = []
         if not back:
             back = self.opts['fileserver_backend']
-        if isinstance(back, six.string_types):
-            back = back.split(',')
-        if all((x.startswith('-') for x in back)):
-            # Only subtracting backends from enabled ones
-            ret = self.opts['fileserver_backend']
-            for sub in back:
-                if '{0}.envs'.format(sub[1:]) in self.servers:
-                    ret.remove(sub[1:])
-                elif '{0}.envs'.format(sub[1:-2]) in self.servers:
-                    ret.remove(sub[1:-2])
         else:
-            for sub in back:
-                if '{0}.envs'.format(sub) in self.servers:
-                    ret.append(sub)
-                elif '{0}.envs'.format(sub[:-2]) in self.servers:
-                    ret.append(sub[:-2])
+            try:
+                back = back.split(',')
+            except AttributeError:
+                back = six.text_type(back).split(',')
+
+        ret = []
+        if not isinstance(back, list):
+            return ret
+
+        try:
+            subtract_only = all((x.startswith('-') for x in back))
+        except AttributeError:
+            pass
+        else:
+            if subtract_only:
+                # Only subtracting backends from enabled ones
+                ret = self.opts['fileserver_backend']
+                for sub in back:
+                    if '{0}.envs'.format(sub[1:]) in self.servers:
+                        ret.remove(sub[1:])
+                    elif '{0}.envs'.format(sub[1:-2]) in self.servers:
+                        ret.remove(sub[1:-2])
+                return ret
+
+        for sub in back:
+            if '{0}.envs'.format(sub) in self.servers:
+                ret.append(sub)
+            elif '{0}.envs'.format(sub[:-2]) in self.servers:
+                ret.append(sub[:-2])
         return ret
 
     def master_opts(self, load):
@@ -502,6 +517,9 @@ class Fileserver(object):
             saltenv = kwargs.pop('env')
         elif 'saltenv' in kwargs:
             saltenv = kwargs.pop('saltenv')
+        if not isinstance(saltenv, six.string_types):
+            saltenv = six.text_type(saltenv)
+
         for fsb in back:
             fstr = '{0}.find_file'.format(fsb)
             if fstr in self.servers:
@@ -528,6 +546,9 @@ class Fileserver(object):
 
         if 'path' not in load or 'loc' not in load or 'saltenv' not in load:
             return ret
+        if not isinstance(load['saltenv'], six.string_types):
+            load['saltenv'] = six.text_type(load['saltenv'])
+
         fnd = self.find_file(load['path'], load['saltenv'])
         if not fnd.get('back'):
             return ret
@@ -551,6 +572,9 @@ class Fileserver(object):
 
         if 'path' not in load or 'saltenv' not in load:
             return ''
+        if not isinstance(load['saltenv'], six.string_types):
+            load['saltenv'] = six.text_type(load['saltenv'])
+
         fnd = self.find_file(salt.utils.locales.sdecode(load['path']),
                 load['saltenv'])
         if not fnd.get('back'):
@@ -576,6 +600,9 @@ class Fileserver(object):
         ret = set()
         if 'saltenv' not in load:
             return []
+        if not isinstance(load['saltenv'], six.string_types):
+            load['saltenv'] = six.text_type(load['saltenv'])
+
         for fsb in self._gen_back(load.pop('fsbackend', None)):
             fstr = '{0}.file_list'.format(fsb)
             if fstr in self.servers:
@@ -604,6 +631,9 @@ class Fileserver(object):
         ret = set()
         if 'saltenv' not in load:
             return []
+        if not isinstance(load['saltenv'], six.string_types):
+            load['saltenv'] = six.text_type(load['saltenv'])
+
         for fsb in self._gen_back(None):
             fstr = '{0}.file_list_emptydirs'.format(fsb)
             if fstr in self.servers:
@@ -632,6 +662,9 @@ class Fileserver(object):
         ret = set()
         if 'saltenv' not in load:
             return []
+        if not isinstance(load['saltenv'], six.string_types):
+            load['saltenv'] = six.text_type(load['saltenv'])
+
         for fsb in self._gen_back(load.pop('fsbackend', None)):
             fstr = '{0}.dir_list'.format(fsb)
             if fstr in self.servers:
@@ -660,6 +693,9 @@ class Fileserver(object):
         ret = {}
         if 'saltenv' not in load:
             return {}
+        if not isinstance(load['saltenv'], six.string_types):
+            load['saltenv'] = six.text_type(load['saltenv'])
+
         for fsb in self._gen_back(load.pop('fsbackend', None)):
             symlstr = '{0}.symlink_list'.format(fsb)
             if symlstr in self.servers:
@@ -687,7 +723,12 @@ class FSChan(object):
         self.kwargs = kwargs
         self.fs = Fileserver(self.opts)
         self.fs.init()
-        self.fs.update()
+        if self.opts.get('file_client', 'remote') == 'local':
+            if '__fs_update' not in self.opts:
+                self.fs.update()
+                self.opts['__fs_update'] = True
+        else:
+            self.fs.update()
         self.cmd_stub = {'ext_nodes': {}}
 
     def send(self, load, tries=None, timeout=None):
