@@ -2,6 +2,7 @@
 '''
 Module for managing dnsmasq
 '''
+from __future__ import absolute_import
 
 # Import salt libs
 import salt.utils
@@ -70,13 +71,19 @@ def set_config(config_file='/etc/dnsmasq.conf', follow=True, **kwargs):
     to the end of the main config file (and not to any includes). If you need
     an option added to a specific include file, specify it as the config_file.
 
+    :param string config_file: config file where settings should be updated / added.
+    :param bool follow: attempt to set the config option inside any file within
+        the ``conf-dir`` where it has already been enabled.
+    :param kwargs: key value pairs that contain the configuration settings that you
+        want set.
+
     CLI Examples:
 
     .. code-block:: bash
 
         salt '*' dnsmasq.set_config domain=mydomain.com
         salt '*' dnsmasq.set_config follow=False domain=mydomain.com
-        salt '*' dnsmasq.set_config file=/etc/dnsmasq.conf domain=mydomain.com
+        salt '*' dnsmasq.set_config config_file=/etc/dnsmasq.conf domain=mydomain.com
     '''
     dnsopts = get_config(config_file)
     includes = [config_file]
@@ -91,7 +98,7 @@ def set_config(config_file='/etc/dnsmasq.conf', follow=True, **kwargs):
             if filename.endswith('#') and filename.endswith('#'):
                 continue
             includes.append('{0}/{1}'.format(dnsopts['conf-dir'], filename))
-    for key in kwargs.keys():
+    for key in kwargs:
         if key in dnsopts:
             if isinstance(dnsopts[key], str):
                 for config in includes:
@@ -111,12 +118,16 @@ def get_config(config_file='/etc/dnsmasq.conf'):
     '''
     Dumps all options from the config file.
 
+    config_file
+        The location of the config file from which to obtain contents.
+        Defaults to ``/etc/dnsmasq.conf``.
+
     CLI Examples:
 
     .. code-block:: bash
 
         salt '*' dnsmasq.get_config
-        salt '*' dnsmasq.get_config file=/etc/dnsmasq.conf
+        salt '*' dnsmasq.get_config config_file=/etc/dnsmasq.conf
     '''
     dnsopts = _parse_dnamasq(config_file)
     if 'conf-dir' in dnsopts:
