@@ -16,9 +16,16 @@ import socket
 import salt.utils
 import salt.utils.decorators as decorators
 import salt.utils.network
-from salt.exceptions import CommandExecutionError
 import salt.utils.validate.net
-from salt.ext.six.moves import range
+from salt.exceptions import CommandExecutionError
+
+# Import 3rd-party libs
+import salt.ext.six as six
+from salt.ext.six.moves import range  # pylint: disable=import-error,no-name-in-module,redefined-builtin
+if six.PY3:
+    import ipaddress
+else:
+    import salt.ext.ipaddress as ipaddress
 
 
 log = logging.getLogger(__name__)
@@ -31,7 +38,6 @@ def __virtual__():
     # Disable on Windows, a specific file module exists:
     if salt.utils.is_windows():
         return False
-
     return True
 
 
@@ -47,22 +53,10 @@ def wol(mac, bcast='255.255.255.255', destport=9):
         salt '*' network.wol 080027136977 255.255.255.255 7
         salt '*' network.wol 08:00:27:13:69:77 255.255.255.255 7
     '''
-    if len(mac) == 12:
-        pass
-    elif len(mac) == 17:
-        sep = mac[2]
-        mac = mac.replace(sep, '')
-    else:
-        raise ValueError('Invalid MAC address')
+    dest = salt.utils.mac_str_to_bytes(mac)
     sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
-    dest = ('\\x' + mac[0:2]).decode('string_escape') + \
-           ('\\x' + mac[2:4]).decode('string_escape') + \
-           ('\\x' + mac[4:6]).decode('string_escape') + \
-           ('\\x' + mac[6:8]).decode('string_escape') + \
-           ('\\x' + mac[8:10]).decode('string_escape') + \
-           ('\\x' + mac[10:12]).decode('string_escape')
-    sock.sendto('\xff' * 6 + dest * 16, (bcast, int(destport)))
+    sock.sendto(b'\xff' * 6 + dest * 16, (bcast, int(destport)))
     return True
 
 
@@ -70,6 +64,9 @@ def ping(host, timeout=False, return_boolean=False):
     '''
     Performs an ICMP ping to a host
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS
+
     CLI Example:
 
     .. code-block:: bash
@@ -91,7 +88,10 @@ def ping(host, timeout=False, return_boolean=False):
         salt '*' network.ping archlinux.org timeout=3
     '''
     if timeout:
-        cmd = 'ping -W {0} -c 4 {1}'.format(timeout, salt.utils.network.sanitize_host(host))
+        if __grains__['kernel'] == 'SunOS':
+            cmd = 'ping -c 4 {1} {0}'.format(timeout, salt.utils.network.sanitize_host(host))
+        else:
+            cmd = 'ping -W {0} -c 4 {1}'.format(timeout, salt.utils.network.sanitize_host(host))
     else:
         cmd = 'ping -c 4 {0}'.format(salt.utils.network.sanitize_host(host))
     if return_boolean:
@@ -215,7 +215,10 @@ def _ppid():
     Return a dict of pid to ppid mappings
     '''
     ret = {}
-    cmd = 'ps -ax -o pid,ppid | tail -n+2'
+    if __grains__['kernel'] == 'SunOS':
+        cmd = 'ps -a -o pid,ppid | tail -n+2'
+    else:
+        cmd = 'ps -ax -o pid,ppid | tail -n+2'
     out = __salt__['cmd.run'](cmd, python_shell=True)
     for line in out.splitlines():
         pid, ppid = line.split()
@@ -286,12 +289,12 @@ def _netstat_bsd():
         except KeyError:
             continue
         # Get the pid-to-ppid mappings for this connection
-        conn_ppid = dict((x, y) for x, y in ppid.items() if x in ptr)
+        conn_ppid = dict((x, y) for x, y in six.iteritems(ppid) if x in ptr)
         try:
             # Master pid for this connection will be the pid whose ppid isn't
             # in the subset dict we created above
             master_pid = next(iter(
-                x for x, y in conn_ppid.items() if y not in ptr
+                x for x, y in six.iteritems(conn_ppid) if y not in ptr
             ))
         except StopIteration:
             continue
@@ -300,6 +303,38 @@ def _netstat_bsd():
     return ret
 
 
+def _netstat_sunos():
+    '''
+    Return netstat information for SunOS flavors
+    '''
+    log.warning('User and program not (yet) supported on SunOS')
+    ret = []
+    for addr_family in ('inet', 'inet6'):
+        # Lookup TCP connections
+        cmd = 'netstat -f {0} -P tcp -an | tail -n+5'.format(addr_family)
+        out = __salt__['cmd.run'](cmd, python_shell=True)
+        for line in out.splitlines():
+            comps = line.split()
+            ret.append({
+                'proto': 'tcp6' if addr_family == 'inet6' else 'tcp',
+                'recv-q': comps[5],
+                'send-q': comps[4],
+                'local-address': comps[0],
+                'remote-address': comps[1],
+                'state': comps[6]})
+        # Lookup UDP connections
+        cmd = 'netstat -f {0} -P udp -an | tail -n+5'.format(addr_family)
+        out = __salt__['cmd.run'](cmd, python_shell=True)
+        for line in out.splitlines():
+            comps = line.split()
+            ret.append({
+                'proto': 'udp6' if addr_family == 'inet6' else 'udp',
+                'local-address': comps[0],
+                'remote-address': comps[1] if len(comps) > 2 else ''})
+
+    return ret
+
+
 def _netstat_route_linux():
     '''
     Return netstat routing information for Linux distros
@@ -431,6 +466,36 @@ def _netstat_route_openbsd():
     return ret
 
 
+def _netstat_route_sunos():
+    '''
+    Return netstat routing information for SunOS
+    '''
+    ret = []
+    cmd = 'netstat -f inet -rn | tail -n+5'
+    out = __salt__['cmd.run'](cmd, python_shell=True)
+    for line in out.splitlines():
+        comps = line.split()
+        ret.append({
+            'addr_family': 'inet',
+            'destination': comps[0],
+            'gateway': comps[1],
+            'netmask': '',
+            'flags': comps[2],
+            'interface': comps[5]})
+    cmd = 'netstat -f inet6 -rn | tail -n+5'
+    out = __salt__['cmd.run'](cmd, python_shell=True)
+    for line in out.splitlines():
+        comps = line.split()
+        ret.append({
+            'addr_family': 'inet6',
+            'destination': comps[0],
+            'gateway': comps[1],
+            'netmask': '',
+            'flags': comps[2],
+            'interface': comps[5]})
+    return ret
+
+
 def netstat():
     '''
     Return information on open ports and states
@@ -442,6 +507,9 @@ def netstat():
     .. versionchanged:: 2014.1.4
         Added support for OpenBSD, FreeBSD, and NetBSD
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS
+
     CLI Example:
 
     .. code-block:: bash
@@ -452,12 +520,18 @@ def netstat():
         return _netstat_linux()
     elif __grains__['kernel'] in ('OpenBSD', 'FreeBSD', 'NetBSD'):
         return _netstat_bsd()
+    elif __grains__['kernel'] == 'SunOS':
+        return _netstat_sunos()
     raise CommandExecutionError('Not yet supported on this platform')
 
 
 def active_tcp():
     '''
-    Return a dict containing information on all of the running TCP connections
+    Return a dict containing information on all of the running TCP connections (currently linux and solaris only)
+
+    .. versionchanged:: 2015.8.4
+
+        Added support for SunOS
 
     CLI Example:
 
@@ -465,13 +539,34 @@ def active_tcp():
 
         salt '*' network.active_tcp
     '''
-    return salt.utils.network.active_tcp()
+    if __grains__['kernel'] == 'Linux':
+        return salt.utils.network.active_tcp()
+    elif __grains__['kernel'] == 'SunOS':
+        # lets use netstat to mimic linux as close as possible
+        ret = {}
+        for connection in _netstat_sunos():
+            if not connection['proto'].startswith('tcp'):
+                continue
+            if connection['state'] != 'ESTABLISHED':
+                continue
+            ret[len(ret)+1] = {
+                'local_addr': '.'.join(connection['local-address'].split('.')[:-1]),
+                'local_port': '.'.join(connection['local-address'].split('.')[-1:]),
+                'remote_addr': '.'.join(connection['remote-address'].split('.')[:-1]),
+                'remote_port': '.'.join(connection['remote-address'].split('.')[-1:])
+            }
+        return ret
+    else:
+        return {}
 
 
 def traceroute(host):
     '''
     Performs a traceroute to a 3rd party host
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS
+
     CLI Example:
 
     .. code-block:: bash
@@ -488,29 +583,32 @@ def traceroute(host):
     out = __salt__['cmd.run'](cmd)
 
     # Parse version of traceroute
-    cmd2 = 'traceroute --version'
-    out2 = __salt__['cmd.run'](cmd2)
-    try:
-        # Linux traceroute version looks like:
-        #   Modern traceroute for Linux, version 2.0.19, Dec 10 2012
-        # Darwin and FreeBSD traceroute version looks like: Version 1.4a12+[FreeBSD|Darwin]
-
-        traceroute_version_raw = re.findall(r'.*[Vv]ersion (\d+)\.([\w\+]+)\.*(\w*)', out2)[0]
-        log.debug('traceroute_version_raw: {0}'.format(traceroute_version_raw))
-        traceroute_version = []
-        for t in traceroute_version_raw:
-            try:
-                traceroute_version.append(int(t))
-            except ValueError:
-                traceroute_version.append(t)
-
-        if len(traceroute_version) < 3:
-            traceroute_version.append(0)
-
-        log.debug('traceroute_version: {0}'.format(traceroute_version))
-
-    except IndexError:
+    if salt.utils.is_sunos():
         traceroute_version = [0, 0, 0]
+    else:
+        cmd2 = 'traceroute --version'
+        out2 = __salt__['cmd.run'](cmd2)
+        try:
+            # Linux traceroute version looks like:
+            #   Modern traceroute for Linux, version 2.0.19, Dec 10 2012
+            # Darwin and FreeBSD traceroute version looks like: Version 1.4a12+[FreeBSD|Darwin]
+
+            traceroute_version_raw = re.findall(r'.*[Vv]ersion (\d+)\.([\w\+]+)\.*(\w*)', out2)[0]
+            log.debug('traceroute_version_raw: {0}'.format(traceroute_version_raw))
+            traceroute_version = []
+            for t in traceroute_version_raw:
+                try:
+                    traceroute_version.append(int(t))
+                except ValueError:
+                    traceroute_version.append(t)
+
+            if len(traceroute_version) < 3:
+                traceroute_version.append(0)
+
+            log.debug('traceroute_version: {0}'.format(traceroute_version))
+
+        except IndexError:
+            traceroute_version = [0, 0, 0]
 
     for line in out.splitlines():
         if ' ' not in line:
@@ -518,7 +616,8 @@ def traceroute(host):
         if line.startswith('traceroute'):
             continue
 
-        if 'Darwin' in str(traceroute_version[1]) or 'FreeBSD' in str(traceroute_version[1]):
+        if 'Darwin' in str(traceroute_version[1]) or 'FreeBSD' in str(traceroute_version[1]) or \
+            __grains__['kernel'] == 'SunOS':
             try:
                 traceline = re.findall(r'\s*(\d*)\s+(.*)\s+\((.*)\)\s+(.*)$', line)[0]
             except IndexError:
@@ -539,8 +638,8 @@ def traceroute(host):
                         'hostname': traceline[1],
                         'ip': traceline[2],
                     }
-                    for x in range(0, len(delays)):
-                        result['ms{0}'.format(x + 1)] = delays[x]
+                    for idx in range(0, len(delays)):
+                        result['ms{0}'.format(idx + 1)] = delays[idx]
             except IndexError:
                 result = {}
 
@@ -577,6 +676,7 @@ def traceroute(host):
     return ret
 
 
+@decorators.which('dig')
 def dig(host):
     '''
     Performs a DNS lookup with dig
@@ -596,6 +696,9 @@ def arp():
     '''
     Return the arp table from the minion
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS
+
     CLI Example:
 
     .. code-block:: bash
@@ -608,12 +711,16 @@ def arp():
         comps = line.split()
         if len(comps) < 4:
             continue
-        if not __grains__['kernel'] == 'OpenBSD':
-            ret[comps[3]] = comps[1].strip('(').strip(')')
-        else:
+        if __grains__['kernel'] == 'SunOS':
+            if ':' not in comps[-1]:
+                continue
+            ret[comps[-1]] = comps[1]
+        elif __grains__['kernel'] == 'OpenBSD':
             if comps[0] == 'Host' or comps[1] == '(incomplete)':
                 continue
             ret[comps[1]] = comps[0]
+        else:
+            ret[comps[3]] = comps[1].strip('(').strip(')')
     return ret
 
 
@@ -643,14 +750,14 @@ def hw_addr(iface):
     return salt.utils.network.hw_addr(iface)
 
 # Alias hwaddr to preserve backward compat
-hwaddr = hw_addr
+hwaddr = salt.utils.alias_function(hw_addr, 'hwaddr')
 
 
 def interface(iface):
     '''
     Return the inet address for a given interface
 
-    .. versionadded:: 2014.7
+    .. versionadded:: 2014.7.0
 
     CLI Example:
 
@@ -665,7 +772,7 @@ def interface_ip(iface):
     '''
     Return the inet address for a given interface
 
-    .. versionadded:: 2014.7
+    .. versionadded:: 2014.7.0
 
     CLI Example:
 
@@ -676,9 +783,23 @@ def interface_ip(iface):
     return salt.utils.network.interface_ip(iface)
 
 
-def subnets():
+def subnets(interfaces=None):
     '''
-    Returns a list of subnets to which the host belongs
+    Returns a list of IPv4 subnets to which the host belongs
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' network.subnets
+        salt '*' network.subnets interfaces=eth1
+    '''
+    return salt.utils.network.subnets(interfaces)
+
+
+def subnets6():
+    '''
+    Returns a list of IPv6 subnets to which the host belongs
 
     CLI Example:
 
@@ -686,7 +807,7 @@ def subnets():
 
         salt '*' network.subnets
     '''
-    return salt.utils.network.subnets()
+    return salt.utils.network.subnets6()
 
 
 def in_subnet(cidr):
@@ -712,7 +833,25 @@ def ip_in_subnet(ip_addr, cidr):
 
         salt '*' network.ip_in_subnet 172.17.0.4 172.16.0.0/12
     '''
-    return salt.utils.network.ip_in_subnet(ip_addr, cidr)
+    return salt.utils.network.in_subnet(cidr, ip_addr)
+
+
+def calc_net(ip_addr, netmask=None):
+    '''
+    Returns the CIDR of a subnet based on
+    an IP address (CIDR notation supported)
+    and optional netmask.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' network.calc_net 172.17.0.5 255.255.255.240
+        salt '*' network.calc_net 2a02:f6e:a000:80:84d8:8332:7866:4e07/64
+
+    .. versionadded:: 2015.8.0
+    '''
+    return salt.utils.network.calc_net(ip_addr, netmask)
 
 
 def ip_addrs(interface=None, include_loopback=False, cidr=None):
@@ -736,14 +875,16 @@ def ip_addrs(interface=None, include_loopback=False, cidr=None):
     else:
         return addrs
 
-ipaddrs = ip_addrs
+ipaddrs = salt.utils.alias_function(ip_addrs, 'ipaddrs')
 
 
-def ip_addrs6(interface=None, include_loopback=False):
+def ip_addrs6(interface=None, include_loopback=False, cidr=None):
     '''
     Returns a list of IPv6 addresses assigned to the host. ::1 is ignored,
     unless 'include_loopback=True' is indicated. If 'interface' is provided,
     then only IP addresses from that interface will be returned.
+    Providing a CIDR via 'cidr="2000::/3"' will return only the addresses
+    which are within that subnet.
 
     CLI Example:
 
@@ -751,10 +892,14 @@ def ip_addrs6(interface=None, include_loopback=False):
 
         salt '*' network.ip_addrs6
     '''
-    return salt.utils.network.ip_addrs6(interface=interface,
+    addrs = salt.utils.network.ip_addrs6(interface=interface,
                                         include_loopback=include_loopback)
+    if cidr:
+        return [i for i in addrs if salt.utils.network.in_subnet(cidr, [i])]
+    else:
+        return addrs
 
-ipaddrs6 = ip_addrs6
+ipaddrs6 = salt.utils.alias_function(ip_addrs6, 'ipaddrs6')
 
 
 def get_hostname():
@@ -776,54 +921,95 @@ def mod_hostname(hostname):
     '''
     Modify hostname
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS (Solaris 10, Illumos, SmartOS)
+
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' network.mod_hostname   master.saltstack.com
+        salt '*' network.mod_hostname master.saltstack.com
     '''
+    #
+    # SunOS tested on SmartOS and OmniOS (Solaris 10 compatible)
+    # Oracle Solaris 11 uses smf, currently not supported
+    #
+    # /etc/nodename is the hostname only, not fqdn
+    # /etc/defaultdomain is the domain
+    # /etc/hosts should have both fqdn and hostname entries
+    #
+
     if hostname is None:
         return False
 
-    hostname_cmd = salt.utils.which('hostname')
+    hostname_cmd = salt.utils.which('hostnamectl') or salt.utils.which('hostname')
+    if salt.utils.is_sunos():
+        uname_cmd = '/usr/bin/uname' if salt.utils.is_smartos() else salt.utils.which('uname')
+        check_hostname_cmd = salt.utils.which('check-hostname')
+
     # Grab the old hostname so we know which hostname to change and then
     # change the hostname using the hostname command
-    o_hostname = __salt__['cmd.run']('{0} -f'.format(hostname_cmd))
+    if hostname_cmd.endswith('hostnamectl'):
+        out = __salt__['cmd.run']('{0} status'.format(hostname_cmd))
+        for line in out.splitlines():
+            line = line.split(':')
+            if 'Static hostname' in line[0]:
+                o_hostname = line[1].strip()
+    elif not salt.utils.is_sunos():
+        o_hostname = __salt__['cmd.run']('{0} -f'.format(hostname_cmd))
+    else:
+        # output: Hostname core OK: fully qualified as core.acheron.be
+        o_hostname = __salt__['cmd.run'](check_hostname_cmd).split(' ')[-1]
 
-    __salt__['cmd.run']('{0} {1}'.format(hostname_cmd, hostname))
+    if hostname_cmd.endswith('hostnamectl'):
+        __salt__['cmd.run']('{0} set-hostname {1}'.format(hostname_cmd, hostname))
+    elif not salt.utils.is_sunos():
+        __salt__['cmd.run']('{0} {1}'.format(hostname_cmd, hostname))
+    else:
+        __salt__['cmd.run']('{0} -S {1}'.format(uname_cmd, hostname.split('.')[0]))
 
     # Modify the /etc/hosts file to replace the old hostname with the
     # new hostname
     host_c = salt.utils.fopen('/etc/hosts', 'r').readlines()
 
-    with salt.utils.fopen('/etc/hosts', 'w') as fh:
+    with salt.utils.fopen('/etc/hosts', 'w') as fh_:
         for host in host_c:
             host = host.split()
 
             try:
                 host[host.index(o_hostname)] = hostname
+                if salt.utils.is_sunos():
+                    # also set a copy of the hostname
+                    host[host.index(o_hostname.split('.')[0])] = hostname.split('.')[0]
             except ValueError:
                 pass
 
-            fh.write('\t'.join(host) + '\n')
+            fh_.write('\t'.join(host) + '\n')
 
     # Modify the /etc/sysconfig/network configuration file to set the
     # new hostname
     if __grains__['os_family'] == 'RedHat':
         network_c = salt.utils.fopen('/etc/sysconfig/network', 'r').readlines()
 
-        with salt.utils.fopen('/etc/sysconfig/network', 'w') as fh:
-            for i in network_c:
-                if i.startswith('HOSTNAME'):
-                    fh.write('HOSTNAME={0}\n'.format(hostname))
+        with salt.utils.fopen('/etc/sysconfig/network', 'w') as fh_:
+            for net in network_c:
+                if net.startswith('HOSTNAME'):
+                    fh_.write('HOSTNAME={0}\n'.format(hostname))
                 else:
-                    fh.write(i)
+                    fh_.write(net)
     elif __grains__['os_family'] == 'Debian':
-        with salt.utils.fopen('/etc/hostname', 'w') as fh:
-            fh.write(hostname + '\n')
+        with salt.utils.fopen('/etc/hostname', 'w') as fh_:
+            fh_.write(hostname + '\n')
     elif __grains__['os_family'] == 'OpenBSD':
-        with salt.utils.fopen('/etc/myname', 'w') as fh:
-            fh.write(hostname + '\n')
+        with salt.utils.fopen('/etc/myname', 'w') as fh_:
+            fh_.write(hostname + '\n')
+
+    # Update /etc/nodename and /etc/defaultdomain on SunOS
+    if salt.utils.is_sunos():
+        with salt.utils.fopen('/etc/nodename', 'w') as fh_:
+            fh_.write(hostname.split('.')[0] + '\n')
+        with salt.utils.fopen('/etc/defaultdomain', 'w') as fh_:
+            fh_.write(".".join(hostname.split('.')[1:]) + '\n')
 
     return True
 
@@ -833,7 +1019,7 @@ def connect(host, port=None, **kwargs):
     Test connectivity to a host using a particular
     port from the minion.
 
-    .. versionadded:: 2014.7
+    .. versionadded:: 2014.7.0
 
     CLI Example:
 
@@ -892,30 +1078,30 @@ def connect(host, port=None, **kwargs):
          _proto,
          garbage,
          _address) = socket.getaddrinfo(address, port, __family, 0, __proto)[0]
+    except socket.gaierror:
+        ret['result'] = False
+        ret['comment'] = 'Unable to resolve host {0} on {1} port {2}'.format(host, proto, port)
+        return ret
 
-        s = socket.socket(family, socktype, _proto)
-        s.settimeout(timeout)
+    try:
+        skt = socket.socket(family, socktype, _proto)
+        skt.settimeout(timeout)
 
         if proto == 'udp':
             # Generate a random string of a
             # decent size to test UDP connection
-            h = hashlib.md5()
-            h.update(datetime.datetime.now().strftime('%s'))
-            msg = h.hexdigest()
-            s.sendto(msg, _address)
-            recv, svr = s.recvfrom(255)
-            s.close()
+            md5h = hashlib.md5()
+            md5h.update(datetime.datetime.now().strftime('%s'))
+            msg = md5h.hexdigest()
+            skt.sendto(msg, _address)
+            recv, svr = skt.recvfrom(255)
+            skt.close()
         else:
-            s.connect(_address)
-            s.shutdown(2)
-    except Exception as e:
+            skt.connect(_address)
+            skt.shutdown(2)
+    except Exception as exc:
         ret['result'] = False
-        try:
-            errno, errtxt = e
-        except ValueError:
-            ret['comment'] = 'Unable to connect to {0} ({1}) on {2} port {3}'.format(host, _address[0], proto, port)
-        else:
-            ret['comment'] = '{0}'.format(errtxt)
+        ret['comment'] = 'Unable to connect to {0} ({1}) on {2} port {3}'.format(host, _address[0], proto, port)
         return ret
 
     ret['result'] = True
@@ -928,6 +1114,8 @@ def is_private(ip_addr):
     Check if the given IP address is a private address
 
     .. versionadded:: 2014.7.0
+    .. versionchanged:: 2015.8.0
+        IPv6 support
 
     CLI Example:
 
@@ -935,7 +1123,7 @@ def is_private(ip_addr):
 
         salt '*' network.is_private 10.0.0.3
     '''
-    return salt.utils.network.IPv4Address(ip_addr).is_private
+    return ipaddress.ip_address(ip_addr).is_private
 
 
 def is_loopback(ip_addr):
@@ -943,6 +1131,8 @@ def is_loopback(ip_addr):
     Check if the given IP address is a loopback address
 
     .. versionadded:: 2014.7.0
+    .. versionchanged:: 2015.8.0
+        IPv6 support
 
     CLI Example:
 
@@ -950,20 +1140,23 @@ def is_loopback(ip_addr):
 
         salt '*' network.is_loopback 127.0.0.1
     '''
-    return salt.utils.network.IPv4Address(ip_addr).is_loopback
+    return ipaddress.ip_address(ip_addr).is_loopback
 
 
 def reverse_ip(ip_addr):
     '''
     Returns the reversed IP address
 
+    .. versionchanged:: 2015.8.0
+        IPv6 support
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' network.reverse_ip 172.17.0.4
     '''
-    return salt.utils.network.IPv4Address(ip_addr).reverse_pointer
+    return ipaddress.ip_address(ip_addr).reverse_pointer
 
 
 def _get_bufsize_linux(iface):
@@ -996,13 +1189,13 @@ def _get_bufsize_linux(iface):
 
 def get_bufsize(iface):
     '''
-    Return network buffer sizes as a dict
+    Return network buffer sizes as a dict (currently linux only)
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' network.getbufsize
+        salt '*' network.get_bufsize eth0
     '''
     if __grains__['kernel'] == 'Linux':
         if os.path.exists('/sbin/ethtool'):
@@ -1048,7 +1241,7 @@ def mod_bufsize(iface, *args, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' network.getBuffers
+        salt '*' network.mod_bufsize tx=<val> rx=<val> rx-mini=<val> rx-jumbo=<val>
     '''
     if __grains__['kernel'] == 'Linux':
         if os.path.exists('/sbin/ethtool'):
@@ -1061,6 +1254,9 @@ def routes(family=None):
     '''
     Return currently configured routes from routing table
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS (Solaris 10, Illumos, SmartOS)
+
     CLI Example:
 
     .. code-block:: bash
@@ -1072,6 +1268,8 @@ def routes(family=None):
 
     if __grains__['kernel'] == 'Linux':
         routes_ = _netstat_route_linux()
+    elif __grains__['kernel'] == 'SunOS':
+        routes_ = _netstat_route_sunos()
     elif __grains__['os'] in ['FreeBSD', 'MacOS', 'Darwin']:
         routes_ = _netstat_route_freebsd()
     elif __grains__['os'] in ['NetBSD']:
@@ -1092,6 +1290,9 @@ def default_route(family=None):
     '''
     Return default route(s) from routing table
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS (Solaris 10, Illumos, SmartOS)
+
     CLI Example:
 
     .. code-block:: bash
@@ -1107,7 +1308,8 @@ def default_route(family=None):
     if __grains__['kernel'] == 'Linux':
         default_route['inet'] = ['0.0.0.0', 'default']
         default_route['inet6'] = ['::/0', 'default']
-    elif __grains__['os'] in ['FreeBSD', 'NetBSD', 'OpenBSD', 'MacOS', 'Darwin']:
+    elif __grains__['os'] in ['FreeBSD', 'NetBSD', 'OpenBSD', 'MacOS', 'Darwin'] or \
+        __grains__['kernel'] == 'SunOS':
         default_route['inet'] = ['default']
         default_route['inet6'] = ['default']
     else:
@@ -1117,6 +1319,8 @@ def default_route(family=None):
     for route in _routes:
         if family:
             if route['destination'] in default_route[family]:
+                if __grains__['kernel'] == 'SunOS' and route['addr_family'] != family:
+                    continue
                 ret.append(route)
         else:
             if route['destination'] in default_route['inet'] or \
@@ -1132,6 +1336,10 @@ def get_route(ip):
 
     .. versionadded:: 2015.5.3
 
+    .. versionchanged:: 2015.8.0
+        Added support for SunOS (Solaris 10, Illumos, SmartOS)
+        Added support for OpenBSD
+
     CLI Example::
 
         salt '*' network.get_route 10.10.10.10
@@ -1146,8 +1354,76 @@ def get_route(ip):
             'destination': ip,
             'gateway': m.group('gateway'),
             'interface': m.group('interface'),
-            'source': m.group('source')}
+            'source': m.group('source')
+        }
 
         return ret
+
+    if __grains__['kernel'] == 'SunOS':
+        # [root@nacl ~]# route -n get 172.16.10.123
+        #   route to: 172.16.10.123
+        #destination: 172.16.10.0
+        #       mask: 255.255.255.0
+        #  interface: net0
+        #      flags: <UP,DONE,KERNEL>
+        # recvpipe  sendpipe  ssthresh    rtt,ms rttvar,ms  hopcount      mtu     expire
+        #       0         0         0         0         0         0      1500         0
+        cmd = '/usr/sbin/route -n get {0}'.format(ip)
+        out = __salt__['cmd.run'](cmd, python_shell=False)
+
+        ret = {
+            'destination': ip,
+            'gateway': None,
+            'interface': None,
+            'source': None
+        }
+
+        for line in out.splitlines():
+            line = line.split(':')
+            if 'route to' in line[0]:
+                ret['destination'] = line[1].strip()
+            if 'gateway' in line[0]:
+                ret['gateway'] = line[1].strip()
+            if 'interface' in line[0]:
+                ret['interface'] = line[1].strip()
+                ret['source'] = salt.utils.network.interface_ip(line[1].strip())
+
+        return ret
+
+    if __grains__['kernel'] == 'OpenBSD':
+        # [root@exosphere] route -n get blackdot.be
+        #   route to: 5.135.127.100
+        #destination: default
+        #       mask: default
+        #    gateway: 192.168.0.1
+        #  interface: vio0
+        # if address: 192.168.0.2
+        #   priority: 8 (static)
+        #      flags: <UP,GATEWAY,DONE,STATIC>
+        #     use       mtu    expire
+        # 8352657         0         0
+        cmd = 'route -n get {0}'.format(ip)
+        out = __salt__['cmd.run'](cmd, python_shell=False)
+
+        ret = {
+            'destination': ip,
+            'gateway': None,
+            'interface': None,
+            'source': None
+        }
+
+        for line in out.splitlines():
+            line = line.split(':')
+            if 'route to' in line[0]:
+                ret['destination'] = line[1].strip()
+            if 'gateway' in line[0]:
+                ret['gateway'] = line[1].strip()
+            if 'interface' in line[0]:
+                ret['interface'] = line[1].strip()
+            if 'if address' in line[0]:
+                ret['source'] = line[1].strip()
+
+        return ret
+
     else:
         raise CommandExecutionError('Not yet supported on this platform')
