@@ -9,9 +9,9 @@ A module to wrap pacman calls, since Arch is the best
     *'pkg.install' is not available*), see :ref:`here
     <module-provider-override>`.
 '''
-from __future__ import absolute_import
 
 # Import python libs
+from __future__ import absolute_import
 import copy
 import logging
 import re
@@ -19,6 +19,8 @@ import re
 # Import salt libs
 import salt.utils
 from salt.exceptions import CommandExecutionError, MinionError
+
+# Import 3rd-party libs
 import salt.ext.six as six
 
 log = logging.getLogger(__name__)
@@ -100,7 +102,7 @@ def latest_version(*names, **kwargs):
     return ret
 
 # available_version is being deprecated
-available_version = latest_version
+available_version = salt.utils.alias_function(latest_version, 'available_version')
 
 
 def upgrade_available(name):
@@ -116,7 +118,7 @@ def upgrade_available(name):
     return latest_version(name) != ''
 
 
-def list_upgrades(refresh=False):
+def list_upgrades(refresh=False, **kwargs):  # pylint: disable=W0613
     '''
     List all available package upgrades on this system
 
@@ -127,14 +129,14 @@ def list_upgrades(refresh=False):
         salt '*' pkg.list_upgrades
     '''
     upgrades = {}
-    options = ['-S', '-p', '-u', '--print-format "%n %v"']
+    cmd = ['pacman', '-S', '-p', '-u', '--print-format', '%n %v']
 
     if refresh:
-        options.append('-y')
+        cmd.append('-y')
 
-    cmd = ('pacman {0}').format(' '.join(options))
-
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
+    call = __salt__['cmd.run_all'](cmd,
+                                   python_shell=False,
+                                   output_loglevel='trace')
 
     if call['retcode'] != 0:
         comment = ''
@@ -148,7 +150,7 @@ def list_upgrades(refresh=False):
     else:
         out = call['stdout']
 
-    for line in iter(out.splitlines()):
+    for line in out.splitlines():
         comps = line.split(' ')
         if len(comps) != 2:
             continue
@@ -596,5 +598,5 @@ def owner(*paths):
     for path in paths:
         ret[path] = __salt__['cmd.run_stdout'](cmd.format(path))
     if len(ret) == 1:
-        return next(ret.itervalues())
+        return next(six.itervalues(ret))
     return ret
