@@ -20,6 +20,8 @@ to the minion config files:
 You can also ask for indexes creation on the most common used fields, which
 should greatly improve performance. Indexes are not created by default.
 
+.. code-block:: yaml
+
     mongo.indexes: true
 
 Alternative configuration values can be used by prefacing the configuration.
@@ -62,9 +64,12 @@ import salt.utils.jid
 import salt.returners
 import salt.ext.six as six
 
+
 # Import third party libs
 try:
     import pymongo
+    version = pymongo.version
+    version = '.'.join(version.split('.')[:2])
     HAS_PYMONGO = True
 except ImportError:
     HAS_PYMONGO = False
@@ -125,17 +130,28 @@ def _get_conn(ret):
     password = _options.get('password')
     indexes = _options.get('indexes', False)
 
-    conn = pymongo.Connection(host, port)
+    # at some point we should remove support for
+    # pymongo versions < 2.3 until then there are
+    # a bunch of these sections that need to be supported
+
+    if float(version) > 2.3:
+        conn = pymongo.MongoClient(host, port)
+    else:
+        conn = pymongo.Connection(host, port)
     mdb = conn[db_]
 
     if user and password:
         mdb.authenticate(user, password)
 
     if indexes:
-        mdb.saltReturns.ensure_index('minion')
-        mdb.saltReturns.ensure_index('jid')
-
-        mdb.jobs.ensure_index('jid')
+        if float(version) > 2.3:
+            mdb.saltReturns.create_index('minion')
+            mdb.saltReturns.create_index('jid')
+            mdb.jobs.create_index('jid')
+        else:
+            mdb.saltReturns.ensure_index('minion')
+            mdb.saltReturns.ensure_index('jid')
+            mdb.jobs.ensure_index('jid')
 
     return conn, mdb
 
@@ -145,27 +161,52 @@ def returner(ret):
     Return data to a mongodb server
     '''
     conn, mdb = _get_conn(ret)
-    col = mdb[ret['id']]
 
     if isinstance(ret['return'], dict):
         back = _remove_dots(ret['return'])
     else:
         back = ret['return']
 
+    if isinstance(ret, dict):
+        full_ret = _remove_dots(ret)
+    else:
+        full_ret = ret
+
     log.debug(back)
-    sdata = {ret['jid']: back, 'fun': ret['fun']}
+    sdata = {'minion': ret['id'], 'jid': ret['jid'], 'return': back, 'fun': ret['fun'], 'full_ret': full_ret}
     if 'out' in ret:
         sdata['out'] = ret['out']
-    col.insert(sdata)
+
+    # save returns in the saltReturns collection in the json format:
+    # { 'minion': <minion_name>, 'jid': <job_id>, 'return': <return info with dots removed>,
+    #   'fun': <function>, 'full_ret': <unformatted return with dots removed>}
+    #
+    # again we run into the issue with deprecated code from previous versions
+
+    if float(version) > 2.3:
+        #using .copy() to ensure that the original data is not changed, raising issue with pymongo team
+        mdb.saltReturns.insert_one(sdata.copy())
+    else:
+        mdb.saltReturns.insert(sdata.copy())
 
 
-def save_load(jid, load):
+def save_load(jid, load, minions=None):
     '''
     Save the load for a given job id
     '''
     conn, mdb = _get_conn(ret=None)
-    col = mdb[jid]
-    col.insert(load)
+    if float(version) > 2.3:
+        #using .copy() to ensure original data for load is unchanged
+        mdb.jobs.insert_one(load.copy())
+    else:
+        mdb.jobs.insert(load.copy())
+
+
+def save_minions(jid, minions):  # pylint: disable=unused-argument
+    '''
+    Included for API consistency
+    '''
+    pass
 
 
 def get_load(jid):
@@ -173,7 +214,8 @@ def get_load(jid):
     Return the load associated with a given job id
     '''
     conn, mdb = _get_conn(ret=None)
-    return mdb[jid].find_one()
+    ret = mdb.jobs.find_one({'jid': jid}, {'_id': 0})
+    return ret['load']
 
 
 def get_jid(jid):
@@ -182,10 +224,12 @@ def get_jid(jid):
     '''
     conn, mdb = _get_conn(ret=None)
     ret = {}
-    for collection in mdb.collection_names():
-        rdata = mdb[collection].find_one({jid: {'$exists': 'true'}})
-        if rdata:
-            ret[collection] = rdata
+    rdata = mdb.saltReturns.find({'jid': jid}, {'_id': 0})
+    if rdata:
+        for data in rdata:
+            minion = data['minion']
+            # return data in the format {<minion>: { <unformatted full return data>}}
+            ret[minion] = data['full_ret']
     return ret
 
 
@@ -195,10 +239,9 @@ def get_fun(fun):
     '''
     conn, mdb = _get_conn(ret=None)
     ret = {}
-    for collection in mdb.collection_names():
-        rdata = mdb[collection].find_one({'fun': fun})
-        if rdata:
-            ret[collection] = rdata
+    rdata = mdb.saltReturns.find_one({'fun': fun}, {'_id': 0})
+    if rdata:
+        ret = rdata
     return ret
 
 
@@ -208,14 +251,8 @@ def get_minions():
     '''
     conn, mdb = _get_conn(ret=None)
     ret = []
-    for name in mdb.collection_names():
-        if len(name) == 20:
-            try:
-                int(name)
-                continue
-            except ValueError:
-                pass
-        ret.append(name)
+    name = mdb.saltReturns.distinct('minion')
+    ret.append(name)
     return ret
 
 
@@ -225,13 +262,8 @@ def get_jids():
     '''
     conn, mdb = _get_conn(ret=None)
     ret = []
-    for name in mdb.collection_names():
-        if len(name) == 20:
-            try:
-                int(name)
-                ret.append(name)
-            except ValueError:
-                pass
+    name = mdb.jobs.distinct('jid')
+    ret.append(name)
     return ret
 
 
