@@ -76,8 +76,9 @@ and then:
 
 Required python modules: psycopg2
 '''
-from __future__ import absolute_import
+
 # Import python libs
+from __future__ import absolute_import
 import json
 import logging
 import re
@@ -86,6 +87,8 @@ import sys
 # Import salt libs
 import salt.utils
 import salt.utils.jid
+import salt.ext.six as six
+
 # Import third party libs
 try:
     import psycopg2
@@ -213,7 +216,7 @@ def returner(load):
         sql, (
             load['fun'],
             load['jid'],
-            json.dumps(unicode(str(load['return']), 'utf-8', 'replace')),
+            json.dumps(six.text_type(str(load['return']), 'utf-8', 'replace')),
             load['id'],
             load.get('success'),
         )
@@ -221,7 +224,7 @@ def returner(load):
     _close_conn(conn)
 
 
-def save_load(jid, clear_load):
+def save_load(jid, clear_load, minions=None):
     '''
     Save the load to the specified jid id
     '''
@@ -253,9 +256,16 @@ def save_load(jid, clear_load):
     _close_conn(conn)
 
 
+def save_minions(jid, minions):  # pylint: disable=unused-argument
+    '''
+    Included for API consistency
+    '''
+    pass
+
+
 def _escape_jid(jid):
     '''
-    Do proper formating of the jid
+    Do proper formatting of the jid
     '''
     jid = str(jid)
     jid = re.sub(r"'*", "", jid)
