@@ -13,7 +13,7 @@ these states. Here is some example SLS:
         - humanname: CentOS-$releasever - Base
         - mirrorlist: http://mirrorlist.centos.org/?release=$releasever&arch=$basearch&repo=os
         - comments:
-            - '#http://mirror.centos.org/centos/$releasever/os/$basearch/'
+            - 'http://mirror.centos.org/centos/$releasever/os/$basearch/'
         - gpgcheck: 1
         - gpgkey: file:///etc/pki/rpm-gpg/RPM-GPG-KEY-CentOS-6
 
@@ -79,9 +79,10 @@ these states. Here is some example SLS:
     ``python-pycurl`` will need to be manually installed if it is not present
     once ``python-software-properties`` is installed.
 
-    On Ubuntu & Debian systems, the ```python-apt`` package is required to be installed.
-    To check if this package is installed, run ``dpkg -l python-software-properties``.
-    ``python-apt`` will need to be manually installed if it is not present.
+    On Ubuntu & Debian systems, the ```python-apt`` package is required to be
+    installed.  To check if this package is installed, run ``dpkg -l
+    python-software-properties``.  ``python-apt`` will need to be manually
+    installed if it is not present.
 
 '''
 from __future__ import absolute_import
@@ -90,9 +91,12 @@ from __future__ import absolute_import
 import sys
 
 # Import salt libs
-from salt.exceptions import CommandExecutionError
+import salt.utils
+
+from salt.exceptions import CommandExecutionError, SaltInvocationError
 from salt.modules.aptpkg import _strip_uri
 from salt.state import STATE_INTERNAL_KEYWORDS as _STATE_INTERNAL_KEYWORDS
+import salt.utils
 
 
 def __virtual__():
@@ -102,41 +106,55 @@ def __virtual__():
     return 'pkg.mod_repo' in __salt__
 
 
-def managed(name, **kwargs):
+def managed(name, ppa=None, **kwargs):
     '''
-    This function manages the configuration on a system that points to the
-    repositories for the system's package manager.
+    This state manages software package repositories. Currently, :mod:`yum
+    <salt.modules.yumpkg>`, :mod:`apt <salt.modules.aptpkg>`, and :mod:`zypper
+    <salt.modules.zypper>` repositories are supported.
+
+    **YUM OR ZYPPER-BASED SYSTEMS**
+
+    .. note::
+        One of ``baseurl`` or ``mirrorlist`` below is required. Additionally,
+        note that this state is not presently capable of managing more than one
+        repo in a single repo file, so each instance of this state will manage
+        a single repo file containing the configuration for a single repo.
 
     name
-        The name of the package repo, as it would be referred to when running
-        the regular package manager commands.
-
-    For yum-based systems, take note of the following configuration values:
+        This value will be used in two ways: Firstly, it will be the repo ID,
+        as seen in the entry in square brackets (e.g. ``[foo]``) for a given
+        repo. Secondly, it will be the name of the file as stored in
+        /etc/yum.repos.d (e.g. ``/etc/yum.repos.d/foo.conf``).
 
     humanname
-        On yum-based systems, this is stored as the "name" value in the .repo
-        file in /etc/yum.repos.d/. On yum-based systems, this is required.
+        This is used as the "name" value in the repo file in
+        ``/etc/yum.repos.d/`` (or ``/etc/zypp/repos.d`` for Suse distros).
 
     baseurl
-        On yum-based systems, baseurl refers to a direct URL to be used for
-        this yum repo.
-        One of baseurl or mirrorlist is required.
+        The URL to a yum repository
 
     mirrorlist
-        a URL which contains a collection of baseurls to choose from. On
-        yum-based systems.
-        One of baseurl or mirrorlist is required.
+        A URL which points to a file containing a collection of baseurls
 
     comments
         Sometimes you want to supply additional information, but not as
         enabled configuration. Anything supplied for this list will be saved
         in the repo configuration with a comment marker (#) in front.
 
-    Additional configuration values, such as gpgkey or gpgcheck, are used
-    verbatim to update the options for the yum repo in question.
+    Additional configuration values seen in yum repo files, such as ``gpgkey`` or
+    ``gpgcheck``, will be used directly as key-value pairs. For example:
+
+    .. code-block:: yaml
+
+        foo:
+          pkgrepo.managed:
+            - humanname: Personal repo for foo
+            - baseurl: https://mydomain.tld/repo/foo/$releasever/$basearch
+            - gpgkey: file:///etc/pki/rpm-gpg/foo-signing-key
+            - gpgcheck: 1
 
 
-    For apt-based systems, take note of the following configuration values:
+    **APT-BASED SYSTEMS**
 
     ppa
         On Ubuntu, you can take advantage of Personal Package Archives on
@@ -167,7 +185,7 @@ def managed(name, **kwargs):
         On apt-based systems this must be the complete entry as it would be
         seen in the sources.list file.  This can have a limited subset of
         components (i.e. 'main') which can be added/modified with the
-        "comps" option.
+        ``comps`` option.
 
         .. code-block:: yaml
 
@@ -175,15 +193,20 @@ def managed(name, **kwargs):
               pkgrepo.managed:
                 - name: deb http://us.archive.ubuntu.com/ubuntu precise main
 
+        .. note::
+
+            The above example is intended as a more readable way of configuring
+            the SLS, it is equivalent to the following:
+
+            .. code-block:: yaml
+
+                'deb http://us.archive.ubuntu.com/ubuntu precise main':
+                  pkgrepo.managed
+
     disabled
         Toggles whether or not the repo is used for resolving dependencies
         and/or installing packages.
 
-    enabled
-        Enables the repository, even if the repository has been disabled, in
-        order for the respective package requiring the repository can be found
-        and installed.
-
     comps
         On apt-based systems, comps dictate the types of packages to be
         installed from the repository (e.g. main, nonfree, ...).  For
@@ -199,7 +222,7 @@ def managed(name, **kwargs):
        for.  (e.g. unstable). This option is rarely needed.
 
     keyid
-       The KeyID of the GPG key to install.  This option also requires
+       The KeyID of the GPG key to install. This option also requires
        the ``keyserver`` option to be set.
 
     keyserver
@@ -222,6 +245,12 @@ def managed(name, **kwargs):
        file.  The consolidate will run every time the state is processed. The
        option only needs to be set on one repo managed by salt to take effect.
 
+    clean_file
+       If set to true, empty file before config repo, dangerous if use
+       multiple sources in one file.
+
+       .. versionadded:: 2015.8.0
+
     refresh_db
        If set to false this will skip refreshing the apt package database on
        debian based systems.
@@ -235,124 +264,175 @@ def managed(name, **kwargs):
            'changes': {},
            'result': None,
            'comment': ''}
-    repo = {}
 
-    # pkg.mod_repo has conflicting kwargs, so move 'em around
+    if 'pkg.get_repo' not in __salt__:
+        ret['result'] = False
+        ret['comment'] = 'Repo management not implemented on this platform'
+        return ret
 
-    if 'name' in kwargs:
-        if 'ppa' in kwargs:
-            ret['result'] = False
-            ret['comment'] = 'You may not use both the "name" argument ' \
-                             'and the "ppa" argument.'
-            return ret
-        kwargs['repo'] = kwargs['name']
-    if 'ppa' in kwargs and __grains__['os'] == 'Ubuntu':
-        # overload the name/repo value for PPAs cleanly
-        # this allows us to have one code-path for PPAs
-        repo_name = 'ppa:{0}'.format(kwargs['ppa'])
-        kwargs['repo'] = repo_name
-    if 'repo' not in kwargs:
-        kwargs['repo'] = name
+    if 'repo' in kwargs:
+        ret['result'] = False
+        ret['comment'] = ('\'repo\' is not a supported argument for this '
+                          'state. The \'name\' argument is probably what was '
+                          'intended.')
+        return ret
 
-    if 'humanname' in kwargs:
-        kwargs['name'] = kwargs['humanname']
+    repo = name
+    if __grains__['os'] == 'Ubuntu':
+        if ppa is not None:
+            # overload the name/repo value for PPAs cleanly
+            # this allows us to have one code-path for PPAs
+            try:
+                repo = ':'.join(('ppa', ppa))
+            except TypeError:
+                repo = ':'.join(('ppa', str(ppa)))
+
+    elif __grains__['os_family'].lower() in ('redhat', 'suse'):
+        if 'humanname' in kwargs:
+            kwargs['name'] = kwargs.pop('humanname')
+        _val = lambda x: '1' if salt.utils.is_true(x) else '0'
+        if 'disabled' in kwargs:
+            if 'enabled' in kwargs:
+                ret['result'] = False
+                ret['comment'] = 'Only one of enabled/disabled is permitted'
+                return ret
+            _reverse = lambda x: '1' if x == '0' else '0'
+            kwargs['enabled'] = _reverse(_val(kwargs.pop('disabled')))
+        elif 'enabled' in kwargs:
+            kwargs['enabled'] = _val(kwargs['enabled'])
+        if 'name' not in kwargs:
+            # Fall back to the repo name if humanname not provided
+            kwargs['name'] = repo
+
+    if kwargs.pop('enabled', None):
+        kwargs['disabled'] = False
+        salt.utils.warn_until(
+            'Boron',
+            'The `enabled` argument has been deprecated in favor of '
+            '`disabled`.'
+        )
 
     for kwarg in _STATE_INTERNAL_KEYWORDS:
         kwargs.pop(kwarg, None)
 
     try:
-        repo = __salt__['pkg.get_repo'](
-                kwargs['repo'],
-                ppa_auth=kwargs.get('ppa_auth', None)
+        pre = __salt__['pkg.get_repo'](
+            repo,
+            ppa_auth=kwargs.get('ppa_auth', None)
         )
     except CommandExecutionError as exc:
         ret['result'] = False
         ret['comment'] = \
-            'Failed to configure repo {0!r}: {1}'.format(name, exc)
+            'Failed to examine repo \'{0}\': {1}'.format(name, exc)
         return ret
 
-    # this is because of how apt-sources works.  This pushes distro logic
+    # This is because of how apt-sources works. This pushes distro logic
     # out of the state itself and into a module that it makes more sense
-    # to use.  Most package providers will simply return the data provided
+    # to use. Most package providers will simply return the data provided
     # it doesn't require any "specialized" data massaging.
     if 'pkg.expand_repo_def' in __salt__:
-        sanitizedkwargs = __salt__['pkg.expand_repo_def'](kwargs)
+        sanitizedkwargs = __salt__['pkg.expand_repo_def'](repo=repo, **kwargs)
     else:
         sanitizedkwargs = kwargs
-    if __grains__['os_family'] == 'Debian':
-        kwargs['repo'] = _strip_uri(kwargs['repo'])
 
-    if repo:
-        notset = False
+    if __grains__['os_family'] == 'Debian':
+        repo = _strip_uri(repo)
+
+    if pre:
+        needs_update = False
         for kwarg in sanitizedkwargs:
-            if kwarg == 'repo':
-                pass
-            elif kwarg not in repo:
-                notset = True
+            if kwarg not in pre:
+                if kwarg == 'enabled':
+                    # On a RedHat-based OS, 'enabled' is assumed to be true if
+                    # not explicitly set, so we don't need to update the repo
+                    # if it's desired to be enabled and the 'enabled' key is
+                    # missing from the repo definition
+                    if __grains__['os_family'] == 'RedHat':
+                        if not salt.utils.is_true(sanitizedkwargs[kwarg]):
+                            needs_update = True
+                    else:
+                        needs_update = True
+                else:
+                    needs_update = True
             elif kwarg == 'comps':
-                if sorted(sanitizedkwargs[kwarg]) != sorted(repo[kwarg]):
-                    notset = True
+                if sorted(sanitizedkwargs[kwarg]) != sorted(pre[kwarg]):
+                    needs_update = True
             elif kwarg == 'line' and __grains__['os_family'] == 'Debian':
                 # split the line and sort everything after the URL
                 sanitizedsplit = sanitizedkwargs[kwarg].split()
                 sanitizedsplit[3:] = sorted(sanitizedsplit[3:])
-                reposplit = repo[kwarg].split()
+                reposplit = pre[kwarg].split()
                 reposplit[3:] = sorted(reposplit[3:])
                 if sanitizedsplit != reposplit:
-                    notset = True
+                    needs_update = True
+                if 'comments' in kwargs:
+                    _line = pre[kwarg].split('#')
+                    if str(kwargs['comments']) not in _line:
+                        needs_update = True
             else:
-                if str(sanitizedkwargs[kwarg]) != str(repo[kwarg]):
-                    notset = True
-        if notset is False:
+                if str(sanitizedkwargs[kwarg]) != str(pre[kwarg]):
+                    needs_update = True
+
+        if not needs_update:
             ret['result'] = True
-            ret['comment'] = ('Package repo {0!r} already configured'
+            ret['comment'] = ('Package repo \'{0}\' already configured'
                               .format(name))
             return ret
+
     if __opts__['test']:
-        ret['comment'] = ('Package repo {0!r} will be configured. This may '
-                          'cause pkg states to behave differently than stated '
-                          'if this action is repeated without test=True, due '
-                          'to the differences in the configured repositories.'
-                          .format(name))
+        ret['comment'] = (
+            'Package repo \'{0}\' will be configured. This may cause pkg '
+            'states to behave differently than stated if this action is '
+            'repeated without test=True, due to the differences in the '
+            'configured repositories.'.format(name)
+        )
         return ret
+
+    # empty file before configure
+    if kwargs.get('clean_file', False):
+        salt.utils.fopen(kwargs['file'], 'w').close()
+
     try:
         if __grains__['os_family'] == 'Debian':
-            __salt__['pkg.mod_repo'](saltenv=__env__, **kwargs)
+            __salt__['pkg.mod_repo'](repo, saltenv=__env__, **kwargs)
         else:
-            __salt__['pkg.mod_repo'](**kwargs)
+            __salt__['pkg.mod_repo'](repo, **kwargs)
     except Exception as exc:
         # This is another way to pass information back from the mod_repo
         # function.
         ret['result'] = False
         ret['comment'] = \
-            'Failed to configure repo {0!r}: {1}'.format(name, exc)
+            'Failed to configure repo \'{0}\': {1}'.format(name, exc)
         return ret
 
     try:
-        repodict = __salt__['pkg.get_repo'](
-            kwargs['repo'], ppa_auth=kwargs.get('ppa_auth', None)
+        post = __salt__['pkg.get_repo'](
+            repo,
+            ppa_auth=kwargs.get('ppa_auth', None)
         )
-        if repo:
+        if pre:
             for kwarg in sanitizedkwargs:
-                if repodict.get(kwarg) != repo.get(kwarg):
-                    change = {'new': repodict[kwarg],
-                              'old': repo.get(kwarg)}
+                if post.get(kwarg) != pre.get(kwarg):
+                    change = {'new': post[kwarg],
+                              'old': pre.get(kwarg)}
                     ret['changes'][kwarg] = change
         else:
-            ret['changes'] = {'repo': kwargs['repo']}
+            ret['changes'] = {'repo': repo}
 
         ret['result'] = True
-        ret['comment'] = 'Configured package repo {0!r}'.format(name)
+        ret['comment'] = 'Configured package repo \'{0}\''.format(name)
     except Exception as exc:
         ret['result'] = False
         ret['comment'] = \
-            'Failed to confirm config of repo {0!r}: {1}'.format(name, exc)
+            'Failed to confirm config of repo \'{0}\': {1}'.format(name, exc)
+
     # Clear cache of available packages, if present, since changes to the
     # repositories may change the packages that are available.
     if ret['changes']:
         sys.modules[
             __salt__['test.ping'].__module__
         ].__context__.pop('pkg._avail', None)
+
     return ret
 
 
@@ -365,6 +445,8 @@ def absent(name, **kwargs):
         The name of the package repo, as it would be referred to when running
         the regular package manager commands.
 
+    **UBUNTU-SPECIFIC OPTIONS**
+
     ppa
         On Ubuntu, you can take advantage of Personal Package Archives on
         Launchpad simply by specifying the user and archive name.
@@ -378,7 +460,7 @@ def absent(name, **kwargs):
     ppa_auth
         For Ubuntu PPAs there can be private PPAs that require authentication
         to access. For these PPAs the username/password can be specified.  This
-        is required for matching if the name format uses the "ppa:" specifier
+        is required for matching if the name format uses the ``ppa:`` specifier
         and is private (requires username/password to access, which is encoded
         in the URI).
 
@@ -388,14 +470,37 @@ def absent(name, **kwargs):
               pkgrepo.absent:
                 - ppa: wolfnet/logstash
                 - ppa_auth: username:password
+
+    keyid
+        If passed, then the GPG key corresponding to the passed KeyID will also
+        be removed.
+
+    keyid_ppa : False
+        If set to ``True``, the GPG key's ID will be looked up from
+        ppa.launchpad.net and removed, and the ``keyid`` argument will be
+        ignored.
+
+        .. note::
+            This option will be disregarded unless the ``ppa`` argument is
+            present.
     '''
     ret = {'name': name,
            'changes': {},
            'result': None,
            'comment': ''}
     repo = {}
-    if 'ppa' in kwargs and __grains__['os'] == 'Ubuntu':
-        kwargs['name'] = kwargs.pop('ppa')
+    if 'ppa' in kwargs and __grains__['os'] in ('Ubuntu', 'Mint'):
+        name = kwargs.pop('ppa')
+        if not name.startswith('ppa:'):
+            name = 'ppa:' + name
+
+    remove_key = any(kwargs.get(x) is not None
+                     for x in ('keyid', 'keyid_ppa'))
+    if remove_key and 'pkg.del_repo_key' not in __salt__:
+        ret['result'] = False
+        ret['comment'] = \
+            'Repo key management is not implemented for this platform'
+        return ret
 
     try:
         repo = __salt__['pkg.get_repo'](
@@ -411,6 +516,7 @@ def absent(name, **kwargs):
         ret['comment'] = 'Package repo {0} is absent'.format(name)
         ret['result'] = True
         return ret
+
     if __opts__['test']:
         ret['comment'] = ('Package repo {0!r} will be removed. This may '
                           'cause pkg states to behave differently than stated '
@@ -418,19 +524,40 @@ def absent(name, **kwargs):
                           'to the differences in the configured repositories.'
                           .format(name))
         return ret
-    __salt__['pkg.del_repo'](repo=name, **kwargs)
+
+    try:
+        __salt__['pkg.del_repo'](repo=name, **kwargs)
+    except (CommandExecutionError, SaltInvocationError) as exc:
+        ret['result'] = False
+        ret['comment'] = exc.strerror
+        return ret
+
     repos = __salt__['pkg.list_repos']()
     if name not in repos:
-        ret['result'] = True
-        ret['changes'] = {'repo': name}
-        ret['comment'] = 'Removed package repo {0}'.format(name)
+        ret['changes']['repo'] = name
+        ret['comment'] = 'Removed repo {0}'.format(name)
+
+        if not remove_key:
+            ret['result'] = True
+        else:
+            try:
+                removed_keyid = __salt__['pkg.del_repo_key'](name, **kwargs)
+            except (CommandExecutionError, SaltInvocationError) as exc:
+                ret['result'] = False
+                ret['comment'] += ', but failed to remove key: {0}'.format(exc)
+            else:
+                ret['result'] = True
+                ret['changes']['keyid'] = removed_keyid
+                ret['comment'] += ', and keyid {0}'.format(removed_keyid)
     else:
         ret['result'] = False
         ret['comment'] = 'Failed to remove repo {0}'.format(name)
+
     # Clear cache of available packages, if present, since changes to the
     # repositories may change the packages that are available.
     if ret['changes']:
         sys.modules[
             __salt__['test.ping'].__module__
         ].__context__.pop('pkg._avail', None)
+
     return ret
