@@ -28,7 +28,8 @@ log = logging.getLogger(__name__)
 def query(key, keyid, method='GET', params=None, headers=None,
           requesturl=None, return_url=False, bucket=None, service_url=None,
           path='', return_bin=False, action=None, local_file=None,
-          verify_ssl=True, location=None, full_headers=False):
+          verify_ssl=True, location=None, full_headers=False, role_arn=None,
+          chunk_size=16384):
     '''
     Perform a query against an S3-like API. This function requires that a
     secret key and the id for that key are passed in. For instance:
@@ -92,10 +93,10 @@ def query(key, keyid, method='GET', params=None, headers=None,
         keyid = salt.utils.aws.IROLE_CODE
 
     data = ''
+    payload_hash = None
     if method == 'PUT':
         if local_file:
-            with salt.utils.fopen(local_file, 'r') as ifile:
-                data = ifile.read()
+            payload_hash = salt.utils.get_hash(local_file, form='sha256')
 
     if not requesturl:
         requesturl = 'https://{0}/{1}'.format(endpoint, path)
@@ -106,9 +107,11 @@ def query(key, keyid, method='GET', params=None, headers=None,
             data=data,
             uri='/{0}'.format(path),
             prov_dict={'id': keyid, 'key': key},
+            role_arn=role_arn,
             location=location,
             product='s3',
             requesturl=requesturl,
+            payload_hash=payload_hash,
         )
 
     log.debug('S3 Request: {0}'.format(requesturl))
@@ -118,17 +121,34 @@ def query(key, keyid, method='GET', params=None, headers=None,
     if not data:
         data = None
 
-    try:
-        result = requests.request(method, requesturl, headers=headers,
+    response = None
+    if method == 'PUT':
+        if local_file:
+            with salt.utils.fopen(local_file, 'r') as data:
+                result = requests.request(method,
+                                          requesturl,
+                                          headers=headers,
+                                          data=data,
+                                          verify=verify_ssl,
+                                          stream=True)
+        response = result.content
+    elif method == 'GET' and local_file and not return_bin:
+        result = requests.request(method,
+                                  requesturl,
+                                  headers=headers,
+                                  data=data,
+                                  verify=verify_ssl,
+                                  stream=True)
+    else:
+        result = requests.request(method,
+                                  requesturl,
+                                  headers=headers,
                                   data=data,
                                   verify=verify_ssl)
         response = result.content
-    except requests.exceptions.HTTPError as exc:
-        log.error('There was an error::')
-        if hasattr(exc, 'code') and hasattr(exc, 'msg'):
-            log.error('    Code: {0}: {1}'.format(exc.code, exc.msg))
-        log.error('    Content: \n{0}'.format(exc.read()))
-        return False
+    if result.status_code >= 400:
+        # On error the S3 API response should contain error message
+        log.debug('    Response content: {0}'.format(response))
 
     log.debug('S3 Response Status Code: {0}'.format(result.status_code))
 
@@ -169,8 +189,9 @@ def query(key, keyid, method='GET', params=None, headers=None,
     # This can be used to save a binary object to disk
     if local_file and method == 'GET':
         log.debug('Saving to local file: {0}'.format(local_file))
-        with salt.utils.fopen(local_file, 'w') as out:
-            out.write(response)
+        with salt.utils.fopen(local_file, 'wb') as out:
+            for chunk in result.iter_content(chunk_size=chunk_size):
+                out.write(chunk)
         return 'Saved to local file: {0}'.format(local_file)
 
     # This can be used to return a binary object wholesale
