@@ -8,6 +8,7 @@ user present with custom homedir
 '''
 
 # Import python libs
+from __future__ import absolute_import
 import os
 import grp
 
@@ -167,6 +168,31 @@ class UserTest(integration.ModuleCase,
         ret = self.run_state('user.absent', name='salt_test')
         self.assertSaltTrueReturn(ret)
 
+    @destructiveTest
+    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
+    def test_user_present_gecos_none_fields(self):
+        '''
+        This is a DESTRUCTIVE TEST it creates a new user on the on the minion.
+
+        It ensures that if no GECOS data is supplied, the fields will be coerced
+        into empty strings as opposed to the string "None".
+        '''
+        ret = self.run_state(
+            'user.present', name='salt_test', fullname=None, roomnumber=None,
+            workphone=None, homephone=None
+        )
+        self.assertSaltTrueReturn(ret)
+
+        ret = self.run_function('user.info', ['salt_test'])
+        self.assertReturnNonEmptySaltType(ret)
+        self.assertEqual('', ret['fullname'])
+        self.assertEqual('', ret['roomnumber'])
+        self.assertEqual('', ret['workphone'])
+        self.assertEqual('', ret['homephone'])
+
+        ret = self.run_state('user.absent', name='salt_test')
+        self.assertSaltTrueReturn(ret)
+
 
 if __name__ == '__main__':
     from integration import run_tests
